#!/usr/bin/env python

from moinformat import make_input, make_linker, make_output, make_parser, \
                       make_serialiser, parse, serialise
from os.path import split
import sys

def main():
    dirname, progname = split(sys.argv[0])
    args = sys.argv[1:]

    # Option values.

    l = filenames = []
    formats = []
    input_encodings = []
    mappings = []
    output_dirs = []
    output_encodings = []
    pagenames = []

    # Flags.

    macros = False
    tree = False

    for arg in args:

        # Detect tree output.

        if arg == "--tree":
            tree = True

        # Detect macro evaluation.

        elif arg == "--macros":
            macros = True

        # Switch to collecting formats.

        elif arg == "--format":
            l = formats
            continue

        # Switch to collecting input encodings.

        elif arg == "--input-encoding":
            l = input_encodings
            continue

        # Switch to collecting mappings.

        elif arg == "--mapping":
            l = mappings
            continue

        # Switch to collecting output locations.

        elif arg == "--output-dir":
            l = output_dirs
            continue

        # Switch to collecting output encodings.

        elif arg == "--output-encoding":
            l = output_encodings
            continue

        # Switch to collecting page names.

        elif arg == "--pagename":
            l = pagenames
            continue

        # Collect options and arguments.

        else:
            l.append(arg)

            # Collect multiple mappings.

            if l is mappings:
                continue

        # Collect filenames normally.

        l = filenames

    format = formats and formats[0] or "html"

    # Derive the page name from the filename if not specified.

    filename = filenames[0]
    pagename = pagenames and pagenames[0] or split(filename)[-1]

    # Derive a proper mapping from the given list of values.

    mapping = {}
    key = None

    for arg in mappings:
        if key is None:
            key = arg
        else:
            mapping[key] = arg
            key = None

    # Obtain output location.

    output_dir = output_dirs and output_dirs[0] or None

    # Obtain encodings.

    input_encoding = input_encodings and input_encodings[0] or None
    output_encoding = output_encodings and output_encodings[0] or None

    # Open the file, parse the content, serialise the document.

    input = make_input("standalone", {"encoding" : input_encoding})

    p = make_parser()
    d = parse(input.readfile(filename), p)

    if macros:
        p.evaluate_macros()

    # Show a document tree for debugging purposes, if requested.

    if tree:
        print d.prettyprint()

    # Otherwise, serialise the document.

    else:
        # Obtain an output context from any specified output details.

        output_context = output_dir and "directory" or "standalone"

        output = make_output(output_context, {"encoding" : output_encoding,
                                              "filename" : output_dir})

        # Obtain a linker using format and pagename details.

        linker = make_linker(format, pagename, mapping)

        # Obtain a serialiser using the configuration.

        serialiser = make_serialiser(format, output, linker)
        print serialise(d, serialiser)

if __name__ == "__main__":
    main()

# vim: tabstop=4 expandtab shiftwidth=4
